import config from "@/api/config.js";
import * as KMap from "@/utils/ol-map/KMap";
import * as util from "@/common/ol_common.js";
import Style from "ol/style/Style";
import Icon from "ol/style/Icon";
import VectorLayer from "ol/layer/Vector.js";
import WKT from "ol/format/WKT.js";
import { reactive } from "vue";
import Point from "ol/geom/Point.js";
import Feature from "ol/Feature";
import { newPoint } from "@/utils/map.js";
import { Fill, Text } from "ol/style";
import { getArea } from "ol/sphere.js";
import * as proj from "ol/proj";
import proj4 from "proj4";
import { register } from "ol/proj/proj4";
proj4.defs(
  "EPSG:38572",
  "+proj=merc +a=6378137 +b=6378137 +lat_ts=0 +lon_0=0 +x_0=0 +y_0=0 +k=1 +units=m +nadgrids=@null +wktext +no_defs +type=crs"
);
register(proj4);
export let mapData = reactive({
  isEdit: false,
  isEditArea: false,
  curPointData: {},
  point: null,
  selectPointArr: [],
});

/**
 * @description 地图层对象
 */
class AuthenticMap {
  constructor() {
    let that = this;
    let vectorStyle = new KMap.VectorStyle();
    this.vectorStyle = vectorStyle;
    // 位置图标
    this.clickPointLayer = new KMap.VectorLayer("clickPointLayer", 9999, {
      style: (f) => {
        const style1 = new Style({
          image: new Icon({
            src: require(`@/assets/images/common/${f.get("icon")}-icon.png`),
            scale: 0.45,
          }),
        });
        const style2 = new Style({
          text: new Text({
            font: "16px sans-serif",
            text: f.get("masterName"),
            offsetY: -40,
            padding: [4, 3, 2, 106],
            fill: new Fill({ color: "#fff" }), // 字体颜色
          }),
        });
        const style3 = new Style({
          image: new Icon({
            src: require(`@/assets/images/common/${f.get("iconBg")}.png`),
            scale: 0.45,
            displacement: [0, 90],
          }),
        });
        return [style1, style2, style3];
      },
    });

    this.locationLayer = new KMap.VectorLayer("locationLayer", 9999, {
      style: () => {
        return new Style({
          image: new Icon({
            src: require("@/assets/images/common/location.png"),
            scale: 0.45,
          }),
        });
      },
    });
  }

  initMap(location, target) {
    let level = 16;
    let coordinate = util.wktCastGeom(location).getFirstCoordinate();
    this.kmap = new KMap.Map(
      target,
      level,
      coordinate[0],
      coordinate[1],
      null,
      6,
      22
    );
    this.kmap.initDraw((e) => {
      mapData.isEdit = true;
      mapData.point = e.feature;
    });
    this.kmap.startDraw();
    this.kmap.addLayer(this.clickPointLayer.layer);
    this.kmap.addLayer(this.locationLayer.layer);
    this.addMapSingerClick();
  }

  //   添加点位
  addPoint(points) {
    const arrPoints = [];
    if (points && points.length > 0) {
      points.forEach((item) => {
        arrPoints.push(newPoint({ ...item, icon: "point",iconBg: "name-bg" }, "point"));
      });
      this.clickPointLayer.source.addFeatures(arrPoints);
    }
  }

  //  设置地图中心点位
  setMapCenter(v) {
    let arrayOfNumbers = [];
    const arrayOfStrings = v.split(",");
    arrayOfNumbers = [arrayOfStrings[1], arrayOfStrings[0]];
    this.kmap.map.getView().setCenter(arrayOfNumbers);
    this.locationLayer.source.clear();
    let point = new Feature(new Point(arrayOfNumbers));
    this.locationLayer.addFeature(point);
  }

  endEdit() {
    this.kmap.endDraw();
    this.kmap.modifyDraw();
    mapData.isEdit = false;
  }

  startDraw() {
    this.kmap.startDraw();
  }

  // 地图点击事件
  addMapSingerClick() {
    let that = this;
    that.kmap.on("singleclick", (evt) => {
      that.kmap.map.forEachFeatureAtPixel(evt.pixel, function (feature, layer) {
        // 点击的图层是否是VectorLayer
        if (
          layer instanceof VectorLayer &&
          layer.get("name") === "clickPointLayer"
        ) {
          const icon = feature.get("icon") === "point" ? "point-act" : "point";
          const iconBg = feature.get("iconBg") === "name-bg" ? "name-bg1" : "name-bg";
          feature.set("icon", icon);
          feature.set("iconBg", iconBg);
          mapData.curPointData = {};
          mapData.point = feature;
          mapData.curPointData = feature.values_;
          that.getSelectPointArr();
          that.kmap.endDraw();
          that.kmap.modifyDraw((e) => {
            mapData.isEdit = false;
            mapData.isEditArea = false;
            mapData.isEditArea = true;
          });
        } else if (
          layer instanceof VectorLayer &&
          layer.get("name") === "defaultPolygonLayer"
        ) {
          that.kmap.endDraw();
          that.kmap.startDraw();
        } else {
          that.kmap.startDraw();
        }
      });
    });
  }

  setPoint(name) {
    const arr = mapData.selectPointArr.filter(item => item.values_.icon==='point-act')
    if(arr.length>0){
        mapData.point = arr[0]
        mapData.point.set("icon", name);
        mapData.point.set("iconBg", 'name-bg');
    }
    if(arr.length===1){
        mapData.selectPointArr = [];
    }
  }

  //添加地块
  setAreaGeometry(geometryArr) {
    let that = this;
    geometryArr.map((item) => {
      that.kmap.setLayerWkt(item.featureWkt);
    });
  }

  //   获取所有选中点位
  getSelectPointArr() {
    const arr = [];
    this.clickPointLayer.source.forEachFeature((f) => {
      if (f.get("icon") === "point-act") {
        arr.push(f);
      }
    });
    mapData.selectPointArr = arr;
  }

  b() {
    // console.log('this.kmap',this.kmap.polygonLayer)
    // console.log('point',mapData.point);
    // let that = this
    // this.clickPointLayer.source.forEachFeature((f) => {
    //     console.log('f',f);
    //     if (f.get("id") === mapData.point.get("id")) {
    //         that.kmap.clickPointLayer.removeFeature(f);
    //         // that.kmap.remove(f)
    //     //   this.kmap.getView().setCenter(extractCoordinates(new WKT().writeFeature(f)));
    //     //   this.kmap.getView().setZoom(16);
    //     }
    //   })
    this.clickPointLayer.source.clear()
    this.kmap.polygonLayer.source.clear()
    this.kmap.removeLayer(this.kmap.polygonLayer.layer)
    this.kmap.removeLayer(this.clickPointLayer.layer)
    // this.kmap.addLayer(this.kmap.polygonLayer.layer)
  }
  c(){
    this.kmap.addLayer(this.kmap.polygonLayer.layer)
    this.kmap.addLayer(this.clickPointLayer.layer)
  }

  //获取地块信息
  getAreaGeometry() {
    const features = this.kmap.getLayerFeatures();
    let geometryArr = [];
    let area = 0;
    // 获取图层上的Polygon，转成geoJson用于回显
    features.forEach((item) => {
      geometryArr.push({ featureWkt: new WKT().writeFeature(item) });
      let geom = item.getGeometry().clone();
      geom.transform(proj.get("EPSG:4326"), proj.get("EPSG:38572"));
      let areaItem = getArea(geom);
      areaItem = (areaItem + areaItem / 2) / 1000;
      area += areaItem;
    });
    return { geometryArr, area: area.toFixed(2) };
  }
}

export default AuthenticMap;
